// ##########################################################################
// ####                                                                  ####
// ####                        Testdaten-Generator                       ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Bag Template Implementation                                      ####
// ####                                                                  ####
// #### Version 1.00, December 18, 1999                                  ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####                     EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://titan.cs.uni-bonn.de/~dreibhol ####
// ####                                                                  ####
// ##########################################################################


#include "bag.h"


// ###### Constructor #######################################################
template<class T> Bag<T>::Bag()
{
   FirstNode.Prev = &FirstNode;
   FirstNode.Next = &FirstNode;
   Pointer        = &FirstNode;
   Count          = 0;
}


// ###### Destructor ########################################################
template<class T> Bag<T>::~Bag()
{
   BagNode<T>* node;
   BagNode<T>* oldNode;
   node = FirstNode.Next;
   while(node != &FirstNode) {
      oldNode = node;      
      node = node->Next;
      delete oldNode;
   }      
   FirstNode.Prev = NULL;
   FirstNode.Next = NULL;
}


// ###### Add element to head of bag ########################################
template<class T> void Bag<T>::addHead(T element)
{
   BagNode<T>* node = new BagNode<T>;
   node->Element    = element;
   node->Prev       = &FirstNode;
   node->Next       = FirstNode.Next;
   node->Next->Prev = node;
   FirstNode.Next   = node;   
   Pointer          = &FirstNode;
   Count++;
}


// ###### Add element to tail of bag ########################################
template<class T> void Bag<T>::addTail(T element)
{
   BagNode<T>* node = new BagNode<T>;
   node->Element    = element;
   node->Prev       = FirstNode.Prev;
   node->Next       = &FirstNode;
   node->Prev->Next = node;
   FirstNode.Prev   = node;   
   Pointer          = &FirstNode;
   Count++;
}


// ###### Remove element from bag ###########################################
template<class T> void Bag<T>::remove(T element)
{
   BagNode<T>* node = findNode(element);
   if(node) {
      removeNode(node);
      Pointer = &FirstNode;
      delete node;
   }
}


// ###### Find BagNode for element ##########################################
template<class T> BagNode<T>* Bag<T>::findNode(const T& element) const
{
   BagNode<T>* node;
   node = FirstNode.Next;
   while(node != &FirstNode) {
      if(node->Element == element)
         return(node);
      node = node->Next;
   }      
   return(NULL);   
}


// ###### Remove given node from bag ########################################
template<class T> void Bag<T>::removeNode(BagNode<T>* node)
{
   node->Prev->Next = node->Next;
   node->Next->Prev = node->Prev;
   Count--;                
}
