// ##########################################################################
// ####                                                                  ####
// ####                        Testdaten-Generator                       ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Dictionary Bag Template                                          ####
// ####                                                                  ####
// #### Version 1.00, December 18, 1999                                  ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####                     EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://titan.cs.uni-bonn.de/~dreibhol ####
// ####                                                                  ####
// ##########################################################################


#ifndef DICTIONARYBAG_H
#define DICTIONARYBAG_H


#include "system.h"


template<class K, class T> struct DictionaryBagNode
{
   DictionaryBagNode<K,T>* Prev;
   DictionaryBagNode<K,T>* Next;
   K                       Key;
   T                       Element;
};


template<class K, class T> class DictionaryBag
{
   // ====== Constructor/Destructor =========================================
   public:
   DictionaryBag();
   ~DictionaryBag();

   // ====== Element add/remove/test functions ==============================
   void addHead(K key, T element);
   void addTail(K key, T element);
   void remove(K key);
   void removeAll(K key);
   void remove(T item);
   inline void   removeHead();
   inline void   removeTail();
   inline bool   contains(const K& key) const;
   inline bool   contains(const T& item) const;
   inline T      find(const K& key) const;
   inline card32 getCount() const;
   
   // ====== DictionaryBag iteration functions =======================================
   inline bool first(T& element);
   inline bool last(T& element);
   inline bool next(T& element);
   inline bool prev(T& element);
   inline bool first(K& key, T& element);
   inline bool last(K& key, T& element);
   inline bool next(K& key, T& element);
   inline bool prev(K& key, T& element);

   // ====== Private data ===================================================
   protected:
   DictionaryBagNode<K,T>* findNode(const K& key) const;
   DictionaryBagNode<K,T>* findNode(const T& item) const;
   void                    removeNode(DictionaryBagNode<K,T>* node);
   
   card32                  Count;
   DictionaryBagNode<K,T>  FirstNode;
   DictionaryBagNode<K,T>* Pointer;
};


#include "dictionarybag.icc"
#include "dictionarybag.cc"


#endif
