// ##########################################################################
// ####                                                                  ####
// ####                  Publikations-Manager Komponenten                ####
// ####                ====================================              ####
// ####                                                                  ####
// #### generator.cc                                                     ####
// ####                                                                  ####
// #### Version 2.00, 19. Dezember 1999                                  ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn                             ####
// ####                     EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                            http://titan.cs.uni-bonn.de/~dreibhol ####
// ####                                                                  ####
// ##########################################################################


#include <fstream.h>
#include <time.h>


#include "system.h"
#include "randomizer.h"
#include "dictionaryset.h"


Randomizer Random;
const card32 maxCache = 1000;


// ###### Menge mit Strings aus Datei einlesen ##############################
DictionarySet<card32,char*>* LadeListe(const char* name)
{
   // Strings aus Datei in Liste einlesen
   DictionarySet<card32,char*>* liste = new DictionarySet<card32,char*>;
   card32                       index = 0;
   ifstream                     is(name);
   if(is.good()) {
      char s[256];
      while(!is.eof()) {
         is.getline(s,sizeof(s));
         if(s[0] != 0x00) {
            liste->add(index,strdup(s));
            index++;
         }
      }
   }
   else {
      cerr << "Eingabedatei nicht gefunden: " << name << endl;
      exit(1);
   }
   if(liste->getCount() < 1) {
      cerr << "Eingabedatei ist leer: " << name << endl;
      exit(1);   
   }
   return(liste); 
}


// ###### Zuflligen String aus Menge whlen ################################
char* ZufallString(const DictionarySet<card32,char*>* liste)
{
   const card32 index = Random.random32() % liste->getCount();
   char* string = liste->find(index);
   if(string == NULL) {
      cerr << "FEHLER: ZufallListe() - NULL-Pointer?!?!" << endl;
      exit(1);
   }
   return(string);
}


// ###### Zuflligen Titel zurckgeben ######################################
char gTitel[256];
char* ZufallTitel(const DictionarySet<card32,char*>* prefix,
                  const DictionarySet<card32,char*>* titel, int typ)
{
   // Mit W'keit 0.25 direkt Titel whlen, ansonsten Prfix+Titel whlen
   const card32 index = Random.random32() % 4;
   if(index == 1)
      strcpy(gTitel,ZufallString(titel));
   else
      sprintf(gTitel,"%s %s",ZufallString(prefix),ZufallString(titel));
   return(gTitel);
}


// ##########################################################################
// #### Datenerzeugung                                                   ####
// ##########################################################################


// ###### Leeren aller vorhandenen Tabellen #################################
void LeereTabellen()
{
   cerr << "Entfernen der alten Tabellen..." << endl;   
   cout << "lock tables PERSON write,INSTANZ write,BUCH write,TECHREP write,ZEITSCHRIFT write,ZUGRIFFSRECHTE write;" << endl;
   cout << "delete from PERSON;" << endl;
   cout << "delete from INSTANZ;" << endl;   
   cout << "delete from BUCH;" << endl;
   cout << "delete from TECHREP;" << endl;
   cout << "delete from ZEITSCHRIFT;" << endl;
   cout << "delete from ZUGRIFFSRECHTE;" << endl;
}


// ###### Optimieren aller vorhandenen Tabellen #############################
void OptimiereTabellen()
{
   cerr << "Optimieren der Tabellen..." << endl; 
   cout << "optimize table PERSON;" << endl;
   cout << "optimize table INSTANZ;" << endl;
   cout << "optimize table BUCH;" << endl;
   cout << "optimize table TECHREP;" << endl;
   cout << "optimize table ZEITSCHRIFT;" << endl;
   cout << "optimize table ZUGRIFFSRECHTE;" << endl;
   cout << "unlock tables;" << endl;
}


// ###### Bibliothekar erzeugen #############################################
void ErstelleBibliothekar()
{
   cerr << "Erstelle Bibliothekar...";
   cout << "insert into PERSON values (1,'','Bibliothekar','admin','555/10001000','555/20002000','puma@puma-server','R9999');" << endl;
   cout << "insert into ZUGRIFFSRECHTE values ";
   cout << "(1,'AUSLEIH'),";
   cout << "(1,'AUSLEIHLISTE'),";
   cout << "(1,'DETAIL'),";
   cout << "(1,'RUECKGABE'),";
   cout << "(1,'WEITERGABE'),";
   cout << "(1,'SUCHEN'),";
   cout << "(1,'PUBL_LOESCHEN'),";
   cout << "(1,'PUBL_HINZUFUEGEN'),";
   cout << "(1,'PUBL_AENDERN'),";
   cout << "(1,'PERSON_LOESCHEN'),";
   cout << "(1,'PERSON_HINZUFUEGEN'),";
   cout << "(1,'PERSON_AENDERN'),";
   cout << "(1,'RECHTE_AENDERN'),";
   cout << "(1,'HISTORY_SEHEN'),";
   cout << "(1,'HISTORY_LOESCHEN');";
   cout << endl;
   cerr << endl;
}


// ###### Benutzer erstellen ################################################
void ErstelleBenutzer(card32 anzahl,
                      DictionarySet<card32,char*>* vornamen,
                      DictionarySet<card32,char*>* prefix,
                      DictionarySet<card32,char*>* namen)
{
   // ====== Personen generieren ============================================
   cerr << anzahl << " Personen generieren..." << endl;
   cout << "insert into PERSON values ";
   char s[256];
   for(card32 i = 0;i < anzahl;i++) {
      // ====== Fortschritt ausgeben ========================================
      if((i % 100) == 0) {
         if((i % 5000) == 0) {
            if(i > 0) cerr << endl;
            sprintf(s,"%6d ",i);
            cerr << s;
         }
         cerr << "#";
         cerr.flush();
      }
            
      // ====== ID, Vorname und Nachname, Passwort ==========================
      cout << "(" << 1000 + i << ",";
      // Mit W'keit 1/3: Vorname + Initial, ansonsten einfacher Vorname
      if((random() % 3) != 1)
         cout << "'" << ZufallString(vornamen) << " " <<  (char)(65 + (Random.random16() % 26)) << ".',";
      else
         cout << "'" << ZufallString(vornamen) << "'" << ",";
      cout << "'" << ZufallString(namen) << "'" << ",";
      cout << "'geheim'" << ",";     
      
      // ====== Telefon, Fax, E-Mail und Raum ===============================
      sprintf(s,"'%03d/%08d','%03d/%08d','test%04d@puma-server','R%04d'",
                random() % 1000,random() % 100000000,random() % 1000,random() % 100000000,i,i+1);  
      cout << s << ")";
      
      // ====== Komma einsetzen oder ggf. neue Befehlszeile beginnen ========
      if(i < (anzahl - 1)) {
         if(((i % maxCache) == 0) && (i < anzahl - 2)) {
            cout << ";" << endl;
            cout << "insert into PERSON values ";
         }
         else
            cout << ",";
      }
   }    
   cout << ";" << endl;
   cerr << endl;


   // ====== Zugriffsrechte eintragen =======================================
   cerr << "Zugriffsrechte eintragen..." << endl;
   cout << "insert into ZUGRIFFSRECHTE values ";
   for(card32 i = 0;i < anzahl;i++) {
      // ====== Fortschritt ausgeben ========================================
      if((i % 100) == 0) {
         if((i % 5000) == 0) {
            if(i > 0) cerr << endl;
            sprintf(s,"%6d ",i);
            cerr << s;
         }
         cerr << "#";
         cerr.flush();
      }

      // ====== Zugriffsrechte vergeben =====================================
      cout << "(" << 1000+i << ",'AUSLEIH'),";
      cout << "(" << 1000+i << ",'AUSLEIHLISTE'),";
      cout << "(" << 1000+i << ",'DETAIL'),";
      cout << "(" << 1000+i << ",'RUECKGABE'),";
      cout << "(" << 1000+i << ",'WEITERGABE'),";
      cout << "(" << 1000+i << ",'SUCHEN')";      

      // ====== Komma einsetzen oder ggf. neue Befehlszeile beginnen ========
      if(i < (anzahl - 1)) {
         if(((i % maxCache) == 0) && (i < anzahl - 2)) {
            cout << ";" << endl;
            cout << "insert into ZUGRIFFSRECHTE values ";
         }
         else
            cout << ",";
      }

   }    
   cout << ";" << endl;
   cerr << endl;
}


// ###### Liste mit Publikationen in Datenbanktabelle schreiben #############
void SchreibePublikationen(const char* tabelle, Bag<char*>* liste)
{
   if(liste->getCount() > 0) {
      cout << "insert into " << tabelle << " values ";
      char* eintrag;
      bool  ok = liste->first(eintrag);
      while(ok == true) {
         liste->removeHead();
         cout << "(" << eintrag << ")";
         if(liste->getCount() > 0) cout << ",";
         free(eintrag);
         ok = liste->first(eintrag);
      }
      cout << ";" << endl;
   }
}


// ###### Publikationen erstellen ###########################################
void ErstellePublikationen(card32 anzahl,
                           card32 maxInstanzenProPublikation,
                           DictionarySet<card32,char*>* vornamen,
                           DictionarySet<card32,char*>* prefix,
                           DictionarySet<card32,char*>* namen,
                           DictionarySet<card32,char*>* titel,
                           DictionarySet<card32,char*>* staedte)
{
   // Publikationen erzeugen   
   cerr << anzahl << " Publikationen generieren..." << endl;
   Bag<char*> buecher;
   Bag<char*> techReps;
   Bag<char*> zeitschriften;
   Bag<char*> instanzen;
   card32     signatur = 1;
   card32     invNr    = 1;
   char       s[256];
   for(card32 i = 0;i < anzahl;i++) {
      // ====== Fortschritt ausgeben ========================================
      if((i % 100) == 0) {
         if((i % 5000) == 0) {
            if(i > 0) cerr << endl;
            sprintf(s,"%6d ",i);
            cerr << s;
         }
         cerr << "#";
         cerr.flush();
      }

      // ====== Zuflligen Publikationstyp erzeugen =========================   
      int typ = Random.random16() % 3;
      switch(typ) {
         case 0:   // Buch einfgen
            sprintf(s,"'S-%06d','%s','%s, %s',%d,'deutsch','ISBN-%04d-%04d','Auflage %d','%s-Verlag'",
               signatur,ZufallTitel(prefix,titel,0),
               ZufallString(namen),ZufallString(vornamen),
               1960+(random() % 50),random() % 10000,random() % 10000, random() % 20,
               ZufallString(namen));
            buecher.addTail(strdup(s));
          break;
         case 1:   // TechRep einfgen
            sprintf(s,"'S-%06d','%s','%s, %s',%d,'deutsch','ISSN-%04d-%04d',%d,'Universitt %s'",
               signatur,ZufallTitel(prefix,titel,1),
               ZufallString(namen),ZufallString(vornamen),
               1960+(random() % 50),random() % 10000,random() % 10000, random() % 20,
               ZufallString(staedte));
            techReps.addTail(strdup(s));
          break;
         case 2:   // Zeitschrift einfgen
            sprintf(s,"'S-%06d','%s',%d,%d,'deutsch','%s-Verlag'",
               signatur,ZufallTitel(prefix,titel,2),
               1+(random() % 51), 1960+(random() % 50),ZufallString(namen));
            zeitschriften.addTail(strdup(s));
          break;         
      }
      
      // ====== Instanzen hinzufgen ========================================
      int anzahl = random() % maxInstanzenProPublikation;
      if(anzahl < 1) anzahl = 1;
      for(int j = 0;j < anzahl;j++) {
         sprintf(s,"'I-%07d','S-%06d','VERFUEGBAR',0",invNr++,signatur);
         instanzen.addTail(strdup(s));
      }
      signatur++;
      
      // ====== Cache ggf. leeren ===========================================
      if(buecher.getCount()       > maxCache) SchreibePublikationen("BUCH",&buecher);
      if(techReps.getCount()      > maxCache) SchreibePublikationen("TECHREP",&techReps);
      if(zeitschriften.getCount() > maxCache) SchreibePublikationen("ZEITSCHRIFT",&zeitschriften);
      if(instanzen.getCount()     > maxCache) SchreibePublikationen("INSTANZ",&instanzen);
   }   
   SchreibePublikationen("BUCH",&buecher);
   SchreibePublikationen("TECHREP",&techReps);
   SchreibePublikationen("ZEITSCHRIFT",&zeitschriften);
   SchreibePublikationen("INSTANZ",&instanzen);   
   cerr << endl;
}




// ###### Hauptprogramm #####################################################
int main(int argc, char* argv[])
{
   // ====== Argumente auswerten ============================================
   if(argc < 4) {
      cerr << "Aufruf: " << argv[0] << " [Anzahl Namen] [Anzahl Publikationen] [Max Instanzen/Publ.]" << endl;
      return(1);
   } 
   int anzahlPersonen = atol(argv[1]);
   int anzahlPublikationen        = atol(argv[2]);
   int maxInstanzenProPublikation = atol(argv[3]);              
   if(anzahlPersonen < 1) anzahlPersonen = 1;
   if(anzahlPublikationen < 1)           anzahlPublikationen        = 1;
   if(maxInstanzenProPublikation < 1)    maxInstanzenProPublikation = 1;
   if(maxInstanzenProPublikation > 1000) maxInstanzenProPublikation = 1000;

   cerr << "Testdaten-Generator - Version 2.0" << endl;
   cerr << "=========================================================" << endl;
   cerr << "Parameter:   Anzahl der Personen        = " << anzahlPersonen             << endl;
   cerr << "             Anzahl der Publikationen   = " << anzahlPublikationen        << endl;
   cerr << "             Max. Instanzen/Publikation = " << maxInstanzenProPublikation << endl;
   cerr << endl;


   // ====== Eingabelisten initialisieren ===================================
   DictionarySet<card32,char*>* vornamen = LadeListe("Vornamen.dat");
   DictionarySet<card32,char*>* namen    = LadeListe("Namen.dat");
   DictionarySet<card32,char*>* prefix   = LadeListe("TitelPrefix.dat");
   DictionarySet<card32,char*>* titel    = LadeListe("Titel.dat");
   DictionarySet<card32,char*>* staedte  = LadeListe("Staedte.dat");


   // ====== Daten erzeugen =================================================
   LeereTabellen();
   ErstelleBibliothekar();
   ErstelleBenutzer(anzahlPersonen,vornamen,prefix,namen);
   ErstellePublikationen(anzahlPublikationen,maxInstanzenProPublikation,
                         vornamen,prefix,namen,titel,staedte);
   OptimiereTabellen();
   
   cerr << "Ich habe fertig!" << endl;
   return(0);
}
